import pathlib

import pandas as pd

import redcap_bridge
from redcap_bridge.utils import map_header_json_to_csv

index_column_header = 'Variable / Field Name'

template_dir = pathlib.Path(redcap_bridge.__file__).parent / 'template_parts'


def validate_project_against_template_parts(project, *templates):
    """
    Validate a built project csv

    Parameters
    ----------
    project: str, buffer
        Filepath of the csv file or csv buffer of the built project
    templates: str, list
        List of file paths of the template part csvs.

    Returns
    ----------
    bool  
        True if the validation was successful
    """

    df_project = pd.read_csv(project)
    # unify column names to conform to csv style
    df_project = df_project.rename(columns=map_header_json_to_csv)
    df_project.index = df_project[index_column_header]
    dfs_templates = []

    if not templates:
        raise ValueError(
            'No template_parts to validate against were specified.')

    for template in templates:
        df_template = pd.read_csv((template_dir / template).with_suffix('.csv'))
        df_template.index = df_template[index_column_header]
        dfs_templates.append(df_template)

    # compare content of template_parts and project
    for template_df in dfs_templates:
        if not all(template_df.columns == df_project.columns):
            raise ValueError(f'Incompatible columns in project '
                             f'({project.columns}) and template '
                             f'({template_df.columns})')

        for i in template_df.index:
            if i not in df_project.index:
                raise ValueError(f'Row {i} is missing in project csv')

            # compare entries of the row and exclude `na` entries
            na_values = template_df.loc[i].isna()
            equal_entries = df_project.loc[i] == template_df.loc[i]
            if not (equal_entries | na_values).all():
                raise ValueError(f'Row {i} differs between project csv and '
                                 f'template')

    print('Validation successful')
    return True


def validate_record_against_template(record_csv, template_csv):
    """
    Validate a RedCap record against a template instrument

    Parameters
    ----------
    record_csv: path
        path to the record csv of that instrument
    template_csv: path
        path to the template csv of an instrument

    Returns
    -------
    True

    Raises
    ------
    ValueError in case of failing validation
    """

    template = pd.read_csv(template_csv)
    record = pd.read_csv(record_csv)

    assert 'Variable / Field Name' in template

    # remove 'record_id' as it is unique to template
    template = template.loc[template['Variable / Field Name'] != 'record_id']

    # Step 1: Assert all fields are preserved
    type_groups = template.groupby(template['Field Type'])
    for group_type in type_groups.groups:

        # check if all options are present for checkbox fields
        if group_type == 'checkbox':
            df_checkboxes = type_groups.get_group(group_type)
            # reduce to only relevant columns
            df_checkboxes = df_checkboxes[['Variable / Field Name',
                                           'Choices, Calculations, OR Slider Labels']]
            for field_name, choices in df_checkboxes.values:
                choice_ids = [c.split(',')[0] for c in choices.split('| ')]

                for cid in choice_ids:
                    if f'{field_name}___{cid}' not in record.columns.values:
                        raise ValueError(f'"{field_name}___{cid}" column '
                                         f'header is missing in record')

        # check that all non-descriptive fields are preserved
        elif group_type != 'descriptive':
            # check if editable field is present
            group_df = type_groups.get_group(group_type)
            for key in group_df['Variable / Field Name'].values:
                if key not in record.columns.values:
                    raise ValueError(
                        f'"{key}" column header is missing in record')

    # Step 2: Check that required fields contain data
    type_groups = template.groupby(template['Required Field?'])
    required_fields_df = type_groups.get_group('y')

    # ignore required 'checkbox' fields
    required_fields_df = required_fields_df.loc[
        required_fields_df['Field Type'] != 'checkbox']
    required_fields = required_fields_df['Variable / Field Name'].values

    for required_field in required_fields:
        empty_record_mask = record[required_field].isnull()
        if empty_record_mask.values.any():
            empty_record = record.loc[empty_record_mask]
            raise ValueError(
                f'records with {empty_record.index.name}='
                f'{empty_record.index.tolist()} do not contain data in '
                f'required field "{required_field}"')

    return True


if __name__ == '__main__':
    pass



import json

import pytest

from redcap_bridge.project_building import build_project, customize_project
from redcap_bridge.project_validation import \
    validate_project_against_template_parts, validate_record_against_template
from redcap_bridge.test_redcap.test_utils import (test_directory,
                                                  initialize_test_dir)

project_dir = test_directory / 'testfiles' / 'TestProject'


@pytest.fixture
def setup_project_csvs():
    # pre-build project csvs based on project definition
    build_project(project_dir / 'structure.csv',
                  project_dir / 'build.csv')
    customize_project(project_dir / 'build.csv',
                      project_dir / 'customizations.csv',
                      output_file=project_dir / 'customized.csv')


def test_validate_project_against_template_parts(initialize_test_dir,
                                                 setup_project_csvs):
    with open(project_dir / 'project.json') as f:
        project_dict = json.load(f)
    template_parts = project_dict['validation']

    validate_project_against_template_parts(project_dir / 'customized.csv',
                                            *template_parts)


def test_validate_record_against_template(initialize_test_dir,
                                          setup_project_csvs):
    record_csv = test_directory / 'testfiles' / 'record.csv'

    # validate against provided project metadata and constructed metadata
    validate_record_against_template(record_csv,
                                     test_directory / 'testfiles' / 'metadata.csv')
    validate_record_against_template(record_csv, project_dir / 'customized.csv')



import json

import pandas as pd
import pytest

import redcap
from redcap_bridge.server_interface import (upload_datadict, download_records,
                                            download_datadict)
from redcap_bridge.test_redcap.test_utils import (test_directory,
                                                  initialize_test_dir)
from redcap_bridge.utils import map_header_csv_to_json

SERVER_CONFIG_YAML = (
        test_directory / 'testfiles' / 'TestProject' / 'project.json').resolve()


@pytest.fixture
def clean_server(initialize_test_dir):
    """
    Put testing server in a defined state: only minimal metadata (instruments)
    and records present
    """
    # replace existing datadict and record at server by default ones
    config = json.load(open(SERVER_CONFIG_YAML, 'r'))

    # TODO: Add step 0: Initialize project (activate surveys and repeating
    #  instruments )
    #  -> requires extension of PyCap by `import_project_info` method

    # first initialize in lazy mode to configure metadata even if server status
    # is corrupted
    redproj = redcap.Project(config['api_url'], config['api_token'], lazy=True)

    default_datadict = pd.DataFrame(data=[['record_id', 'my_first_instrument',
                                           'text', 'Record ID'] + [''] * 14],
                                    columns=map_header_csv_to_json)
    redproj.import_metadata(default_datadict, format='csv')

    # second initialize in non-lazy mode to configure records
    redproj = redcap.Project(config['api_url'], config['api_token'], lazy=False)
    default_records = pd.DataFrame(columns=['record_id',
                                            'my_first_instrument_complete'])
    redproj.import_records(default_records,
                           format='csv', return_format='json',
                           overwrite="overwrite")


def test_upload_datadict(clean_server, initialize_test_dir):
    """
    Test uploading a survey definition (datadict) csv to the server
    """
    # uploading metadata csv files from testfile dataset and compare to
    # return value of upload
    metadata_csv = test_directory / 'testfiles' / 'metadata.csv'
    res = upload_datadict(metadata_csv, SERVER_CONFIG_YAML)

    # count number of non-empty lines in original csv
    with open(metadata_csv) as f:
        lines = f.readlines()
        exp = len(lines) - 1  # header row does not generate a record

    assert exp == res

def test_upload_records(clean_server, initialize_test_dir):
    """
    Test upload of records to the server

    TODO: Finally this test should test the corresponding redcap_bridge
    `upload_records` method instead of pycap itself
    """
    # upload data records
    config = json.load(open(SERVER_CONFIG_YAML, 'r'))
    redproj = redcap.Project(config['api_url'], config['api_token'], lazy=False)

    upload_datadict(test_directory / 'testfiles' / 'metadata.csv',
                    SERVER_CONFIG_YAML)

    uploaded_records = pd.read_csv(test_directory / 'testfiles' / 'record.csv',
                                   index_col=0, dtype='str')
    redproj.import_records(uploaded_records, format='csv', overwrite='overwrite')


def test_download_records(clean_server, initialize_test_dir):
    """
    Download datadict from server and compare to previously uploaded datadict
    """
    # uploading metadata csv files from testfile dataset and compare to
    # return value of upload
    original_metadata_csv = test_directory / 'testfiles' / 'metadata.csv'
    upload_datadict(original_metadata_csv, SERVER_CONFIG_YAML)


    downloaded_metadata_csv = test_directory / 'testfiles' / 'metadata_downloaded.csv'
    download_datadict(downloaded_metadata_csv, SERVER_CONFIG_YAML)


    import csv
    original_reader = csv.reader(open(original_metadata_csv))
    download_reader = csv.reader(open(downloaded_metadata_csv))

    # comparing headers
    original_header = original_reader.__next__()
    downloaded_header = download_reader.__next__()
    for oh, dh in zip(original_header, downloaded_header):
        # translate header of uploaded csv to follow csv standard
        assert map_header_csv_to_json[oh] == dh

    # compare content
    for oline, dline in zip(original_reader, download_reader):
        assert oline == dline



import git
import pathlib

# TODO: This can be extracted via the RedCap API
header_json = ['field_name', 'form_name', 'section_header', 'field_type',
               'field_label', 'select_choices_or_calculations', 'field_note',
               'text_validation_type_or_show_slider_number',
               'text_validation_min', 'text_validation_max', 'identifier',
               'branching_logic', 'required_field', 'custom_alignment',
               'question_number', 'matrix_group_name', 'matrix_ranking',
               'field_annotation']
header_csv = ['Variable / Field Name', 'Form Name', 'Section Header',
              'Field Type', 'Field Label',
              'Choices, Calculations, OR Slider Labels', 'Field Note',
              'Text Validation Type OR Show Slider Number',
              'Text Validation Min', 'Text Validation Max', 'Identifier?',
              'Branching Logic (Show field only if...)', 'Required Field?',
              'Custom Alignment', 'Question Number (surveys only)',
              'Matrix Group Name', 'Matrix Ranking?', 'Field Annotation']

map_header_json_to_csv = {json: csv for json, csv in zip(header_json,
                                                         header_csv)}
map_header_csv_to_json = {csv: json for csv, json in zip(header_csv,
                                                         header_json)}

def get_repo_state(path):
    """
    Extract the latest commit hash of a git repository

    Args:
        path: Path to the git repository

    Returns:
        2-tuple (str, bool)
            latest commit id
            repo status: True if repository is in a clean state

    Raises:
        ValueError: if path is not part of a git repository
    """

    repo_root = None
    # find repository root folder
    path = pathlib.Path(path)
    for parent in [path] + list(path.parents):
        if (parent / '.git').exists():
            repo_root = parent
            break

    if repo_root is None:
        return '', None

    repo = git.Repo.init(str(repo_root))
    clean = not repo.is_dirty()
    try:
        commit_hash = repo.head.commit.hexsha
    except ValueError as e:
        commit_hash = ''
        clean = False

    return commit_hash, clean


import os
import pathlib
import json
import tempfile

from redcap_bridge.project_building import build_project, customize_project
from redcap_bridge.project_validation import validate_project_against_template_parts
from redcap_bridge.server_interface import upload_datadict, download_records

def setup_project(proj_folder, working_dir=None, include_provenance=True):
    """
    Build a project csv from its specifications and setup on the server

    Parameters
    ----------
        proj_folder: (path)
            folder containing the project specification files `project.json`,
            `structure.csv` and `customizations.csv`
        working_dir: (path)
            directory in which to store temporarily generated project files
        include_provenance: (bool)
            include hidden provenance information in project csv.
            Default: True
    """

    if working_dir is None:
        working_dir = tempfile.TemporaryDirectory(prefix='redcap_bridge_').name

    working_dir = pathlib.Path(working_dir)
    proj_folder = pathlib.Path(proj_folder)

    if not working_dir.exists():
        os.mkdir(working_dir)

    with open(proj_folder / 'project.json') as f:
        proj_conf = json.load(f)

    build_project(proj_folder / 'structure.csv', working_dir / 'build.csv',
                  include_provenance=include_provenance)
    customize_project(working_dir / 'build.csv',
                      proj_folder / 'customizations.csv',
                      output_file=working_dir / 'customized.csv')
    validate_project_against_template_parts(working_dir / 'customized.csv',
                                            *proj_conf['validation'])

    upload_datadict(working_dir / 'customized.csv',
                    proj_folder / 'project.json')


import json

import pandas as pd

from redcap_bridge.project_building import (build_project, customize_project,
                                            extract_customization)
from redcap_bridge.test_redcap.test_utils import (test_directory,
                                                  initialize_test_dir)

project_dir = test_directory / 'testfiles' / 'TestProject'


def test_build_and_customize_project(initialize_test_dir):
    """
    Test building and customizing a project based on its project specifications
    """
    # Running test project build
    build_project(project_dir / 'structure.csv',
                  project_dir / 'build.csv')
    assert (project_dir / 'build.csv').exists()

    # Running test project customization
    customize_project(project_dir / 'build.csv',
                      project_dir / 'customizations.csv',
                      output_file=project_dir / 'customized.csv')
    assert (project_dir / 'customized.csv').exists()


def test_project_build_with_provenance():
    # Running test project build
    build_project(project_dir / 'structure.csv',
                  project_dir / 'build.csv')

    exp_prov_lines = ["provenance_diglabtools_commit,diglabform,,descriptive",
                      "provenance_diglabtools_version,diglabform,,descriptive",
                      "provenance_redcap_forms_commit,diglabform,,descriptive"]

    with open(project_dir / 'build.csv', 'r') as f:
        lines = f.readlines()
        for lid, line in enumerate(lines[-3:]):
            assert line.startswith(exp_prov_lines[lid])
            assert line.endswith(',@HIDDEN\n')


def test_extract_customization(initialize_test_dir):
    """
    Test extracting the customizations.csv from a complete project build and
    known included template parts
    """
    # build and customize project
    build_project(project_dir / 'structure.csv',
                  project_dir / 'build.csv')
    customize_project(project_dir / 'build.csv',
                      project_dir / 'customizations.csv',
                      output_file=project_dir / 'customized.csv')

    # get template parts to compare to
    with open(project_dir / 'project.json') as f:
        project_dict = json.load(f)
    template_parts = project_dict['validation']

    # extract customization
    extract_customization(project_dir / 'customized.csv',
                          project_dir / 'extracted_customizations.csv',
                          *template_parts)

    original = pd.read_csv(project_dir / 'customizations.csv')
    extracted = pd.read_csv(project_dir / 'extracted_customizations.csv')

    assert original.equals(extracted)



import pytest
from redcap_bridge.project_control import setup_project
from redcap_bridge.test_redcap.test_utils import (test_directory,
                                                  initialize_test_dir)

def test_setup_project(initialize_test_dir):
    """
    Test building a complete project from its specifications and uploading it
    to the server
    """
    working_dir = test_directory / 'working_dir'
    working_dir.mkdir(exist_ok=True)

    setup_project(test_directory / 'testfiles' / 'TestProject',
                  working_dir=working_dir)




import pathlib

import warnings

import numpy as np
import pandas as pd

import redcap_bridge
from redcap_bridge.project_validation import \
    validate_project_against_template_parts
from redcap_bridge.utils import map_header_json_to_csv, get_repo_state

redcap_bridge_dir = pathlib.Path(redcap_bridge.__file__).parent
template_dir = redcap_bridge_dir / 'template_parts'


def build_project(project_csv, output_file=None, include_provenance=True):
    """
    Build a complete RedCap Instrument CSV from a set of template_parts and a
    project csv file.

    Parameters
    ----------
    project_csv: str
        Filepath of the project csv file
    
    output_file: str, None
        Filepath of the resulting, complete project csv (with inserted
        template_parts. If None, the content is only returned and not saved.
        Default: None

    include_provenance: bool
        If True, include hidden fields in the project csv that contain
        git commits of source files
        Default: True

    Returns
    -------
        (list) list containing the lines of the complete project definition
        including the template content
    """
    output = []
    used_template = []
    if isinstance(project_csv, str):
        project_csv = pathlib.Path(project_csv)

    with open(project_csv) as f:
        for line in f.readlines():
            # if line only contains reference then include reference here
            if line[0] == '{' and line[-2:] == '}\n' and not (',' in line):
                template_name = pathlib.Path(line[1:-2]).with_suffix('.csv')
                used_template.append(template_name)
                include_file = (template_dir / template_name).resolve()

                if not include_file.exists():
                    raise ValueError(f'Can not build project {project_csv}. '
                                     f'Included template {include_file} does '
                                     f'not exist')

                with open(include_file, 'r') as f_include:
                    header = f_include.readline()
                    if header != output[0]:
                        raise ValueError('Project csv and template do not share'
                                         'the same header. Compare '
                                         f'{project_csv} and {include_file}')
                    output.extend(f_include.readlines())
            else:
                output.append(line)

        if include_provenance:

            # collect provenance infos
            diglabtools_hash, diglabtools_state = get_repo_state(redcap_bridge_dir.parent)
            diglabtools_version = open(redcap_bridge_dir.parent / 'VERSION').read()
            project_conf_repo_hash, project_conf_repo_state = get_repo_state(project_csv)

            if not diglabtools_state:
                warnings.warn('DigLabTools repository is in non-clean state.')
            if project_conf_repo_state is None:
                warnings.warn('Project configuration is not part of a version control repository.')
            elif not project_conf_repo_state:
                warnings.warn('Project configuration repository is in non-clean state.')

            # adding hidden rows to capture provenance info
            include_file = template_dir / '_provenance.csv'
            with open(include_file, 'r') as f_include:
                header = f_include.readline()
                if header != output[0]:
                    raise ValueError('Project csv and template do not share'
                                     'the same header. Compare '
                                     f'{project_csv} and {include_file}')

                for l in f_include.readlines():
                    complete_line = l.format(diglabtools_hash=diglabtools_hash,
                                             diglabtools_version=diglabtools_version,
                                             redcap_forms_hash=project_conf_repo_hash)
                    output.append(complete_line)

    if output_file:
        with open(output_file, 'w') as f:
            f.writelines(output)
    # Print of templates used and total number of fields
    print(f"\nUsed template: ", end='')
    for template in used_template:
        print(f"{template} ", end='')
    print(f"with total of : {len(output)} fields", end='')

    return output


def customize_project(project_built_csv, customization_csv, output_file=None):
    """
    Fill in a built project csv with project specific customizations.

    This can be used to e.g. change the default values of fields or customize
    the list of experimenters to be selected

    Parameters
    ----------
    project_built_csv: str
        The filepath to the csv containing the built project (see also `build_project`)
    customization_csv: str
        The filepath to the csv containing the project customizations
    output_file: str
        The path to save the combined csv. Default: None

    Returns
    -------
    dataframe
        pandas dataframe csv representation of the customized project definition
    """

    # Loading project and customization data
    project_df = pd.read_csv(project_built_csv, dtype=str)
    project_df.index = project_df['Variable / Field Name']
    customization_df = pd.read_csv(customization_csv, dtype=str)
    customization_df.index = customization_df['Variable / Field Name']

    # Utility functions for combining dataframes and series
    def combine_values(value1, value2):
        """
        Combine values by overwriting `na` with data if possible.

        Raises:
            ValueError if value1 and value2 contain contradicting data
        """
        # return value if identical or both not available
        if value1 == value2 or (pd.isna(value1) and pd.isna(value2)):
            return value1
        elif pd.isna(value1) and not pd.isna(value2):
            return value2
        elif not pd.isna(value1) and pd.isna(value2):
            return value1
        else:
            raise ValueError(f'Can not merge contradicting entries {value1} '
                             f'and {value2}')

    combine_series = lambda x, y: x.combine(y, func=combine_values)

    # combine project and customization
    combined_df = project_df.combine(customization_df, combine_series)

    # restore the original order of columns and rows
    combined_df = combined_df.reindex(columns=project_df.columns,
                                      index=project_df.index)

    if output_file is not None:
        combined_df.to_csv(output_file, index=False)

    # print number of customizing field
    print(f"\nCustomizing: {len(customization_df)} fields\n", end='')

    # print the field name
    print(f"The following fields were customized:")
    if(len(customization_df)) < 10:
        print(f"{customization_df['Variable / Field Name'].to_csv(index=False)} ", end='')
    else:
        print(f"{customization_df['Variable / Field Name'].values}")

    return combined_df


def extract_customization(project_csv, export_custom_csv, *template_parts):
    """
    Extract custom parts of a project data dict by subtracting template parts

    Parameters
    ----------
        project_csv: path
            path to the complete project csv file
        export_custom_csv: path
            path to store the resulting customization csv file
        *template_parts: (list)
            list of template parts included in the project

    Returns
    -------

    """

    # ensure the project csv is compatible with the templates
    validate_project_against_template_parts(project_csv, *template_parts)

    custom_df = pd.read_csv(project_csv, dtype=str)
    custom_df = custom_df.rename(columns=map_header_json_to_csv)
    custom_df.index = custom_df['Variable / Field Name']
    custom_df.drop('Variable / Field Name', axis=1, inplace=True)

    for template_part in template_parts:
        template_path = (template_dir / template_part).with_suffix('.csv')
        template_df = pd.read_csv(template_path, dtype=str)
        template_df.index = template_df['Variable / Field Name']
        template_df.drop('Variable / Field Name', axis=1, inplace=True)

        mask = ~ template_df.isna()
        custom_df[mask] = np.nan
        # keep variable / field name column entries

    # remove custom structural fields (from structure.csv)
    mask = ((custom_df['Form Name'].isna()) & (custom_df['Field Type'].isna()) &
            (custom_df['Field Label'].isna()))
    custom_df = custom_df.loc[mask]

    # remove rows and columns that don't contain custom infos
    custom_df.dropna(axis=0, how='all', inplace=True)
    custom_df.dropna(axis=1, how='all', inplace=True)

    # save the resulting customization csv
    if export_custom_csv is not None:
        custom_df.to_csv(export_custom_csv, index=True)


if __name__ == '__main__':
    pass



import json

import pandas as pd

import redcap
from redcap_bridge.utils import map_header_csv_to_json


def upload_datadict(csv_file, server_config_json):
    """
   Parameters
   ----------
   csv_file: str
       Path to the csv file to be used as data dictionary
   server_config_json: str
       Path to the json file containing the redcap url and api token

    Returns:
    ----------

    Returns:
        (int): The number of uploaded fields
    """

    df = pd.read_csv(csv_file, dtype=str)
    df.rename(columns=map_header_csv_to_json, inplace=True)

    # Upload csv using pycap
    config = json.load(open(server_config_json, 'r'))
    redproj = redcap.Project(config['api_url'], config['api_token'], lazy=False)
    n = redproj.import_metadata(df, format='csv', return_format='json')
    return n


def download_datadict(save_to, server_config_json, format='csv'):
    """
    Parameters
    ----------
    save_to: str
        Path where to save the retrieved data dictionary
    server_config_json: str
        Path to the json file containing the redcap url and api token
    format:  'csv', 'json'
        Format of the retrieved data dictionary

            Format of the retrieved data dictionary

    """

    config = json.load(open(server_config_json, 'r'))

    redproj = redcap.Project(config['api_url'], config['api_token'], lazy=False)
    data_dict = redproj.export_metadata(format=format)

    if format == 'csv':
        with open(save_to, 'w') as save_file:
            save_file.writelines(data_dict)
    elif format == 'df':
        data_dict.to_csv(save_to)
    elif format == 'json':
        with open(save_to, 'w') as save_file:
            json.dump(data_dict, save_file)
    else:
        raise ValueError(f'Unknown format {format}. Valid formats are "csv" '
                         f'and "json".')


def download_records(save_to, server_config_json, format='csv'):
    """
    Download records from the redcap server.

    Parameters
    ----------
    save_to: str
        Path where to save the retrieved records csv
    server_config_json: str
        Path to the json file containing the redcap url and api token
    format: 'csv', 'json'
        Format of the retrieved records
    """
    config = json.load(open(server_config_json, 'r'))

    redproj = redcap.Project(config['api_url'], config['api_token'], lazy=False)
    records = redproj.export_records(format=format)

    if format == 'csv':
        with open(save_to, 'w') as save_file:
            save_file.writelines(records)
    elif format == 'df':
        records.to_csv(save_to)
    elif format == 'json':
        with open(save_to, 'w') as save_file:
            json.dump(records, save_file)
    else:
        raise ValueError(f'Unknown format {format}. Valid formats are "csv" '
                         f'and "json".')


def get_json_csv_header_mapping(server_config_json):
    """
    Retruns
    ----------
    dict: 
        Mapping of json to csv headers

    Returns:
        dict: Mapping of json to csv headers
    """

    # TODO: This function should replace utils.py/map_header_json_to_csv
    raise NotImplementedError()


if __name__ == '__main__':
    pass



import os
import pathlib
import shutil
import tempfile

import pytest

test_directory = pathlib.Path(tempfile.gettempdir()) / 'diglabtools_testfiles'


@pytest.fixture
def initialize_test_dir(clean=True):
    """
    Create main test folder if required and add test files

    Parameters
    ----------
    clean: (bool)
        Remove test folder first in case it exists.

    Returns
    -------
    path
        path of the test directory
    """
    if clean and os.path.exists(test_directory):
        shutil.rmtree(test_directory)
    if not os.path.exists(test_directory):
        os.mkdir(test_directory)

    # initialize test files
    packaged_testfolder = pathlib.Path(__file__).parent / 'testfiles'
    shutil.copytree(packaged_testfolder, test_directory / 'testfiles')

    return test_directory



"""
Provide CLI for the main functionalities of the redcap bridge
"""

from redcap_bridge.server_interface import download_records


def main(command_line=None):
    # importing required modules
    import argparse

    # create a parser object
    parser = argparse.ArgumentParser(description="A simple Python interface for RedCap")

    parser.add_argument(
        '--debug',
        action='store_true',
        help='Print debug info'
    )

    subparsers = parser.add_subparsers(dest='command')
    download = subparsers.add_parser('download', help='Downloads the data records')
    download.add_argument("destination", nargs=1, metavar='destination', type=str,
                          help="The destination filename.")
    download.add_argument("config_json", nargs=1, metavar='config_json', type=str,
                          help="The json configuration file of the project")
    download.add_argument("--format", type=str, nargs=1, metavar='format',
                          help="Format to store the data (json/csv)")

    # parse arguments
    args = parser.parse_args(command_line)

    if args.debug:
        print("debug: " + str(args))
    if args.command == 'download':
        download_records(args.destination[0], args.config_json[0])


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import json

from setuptools import setup, find_packages
import os

# Extract central version information
with open(os.path.join(os.path.dirname(__file__), "VERSION")) as version_file:
    version = version_file.read().strip()

with open('requirements.txt') as f:
    requires = f.read().splitlines()

with open('README.md') as f:
    long_description = f.read()

setup(
    name="DigLabTools",
    version=version,
    packages=find_packages(),
    package_data={
        # If any package contains *.json or *.csv files, include them:
        "": ["*.json", '*.csv', '*.zip'],
    },
    data_files=[('.', ['VERSION', 'README.md', 'requirements.txt'])],
    author="Julia Sprenger, Jeremy Garcia",
    description="Tools to interact with the DigLab metadata collection standard",
    long_description_content_type="text/markdown",
    long_description=long_description,
    license='MIT',
    install_requires=requires,
    include_package_data=True,
    python_requires='>=3.6',
    extras_require={
        'test': ['pytest']
    },
    entry_points={
        "console_scripts": [
            "RedCapBridge=redcap_bridge.cli:main",
        ],
    }
)
